/*
 * File name: Matrix.java
 * Date:      2014/11/04 13:38
 * Author:    Jan Faigl
 */
package cz.cvut.fel.pr1;

public class Matrix {

    protected final double[][] values;
    protected final int rows;
    protected final int cols;

    public Matrix(int rows, int cols) {
        if (rows <= 0 || cols <= 0) {
            throw new IllegalArgumentException();
        }
        this.rows = rows;
        this.cols = cols;
        values = new double[rows][];
        for (int i = 0; i < rows; i++) {
            values[i] = new double[cols];
        }
    }

    public Matrix(Matrix m) {
        this(m.rows, m.cols);
    }

    protected Matrix createMatrix(Matrix matrix) {
        return createMatrix(matrix.rows, matrix.cols);
    }

    protected Matrix createMatrix(int rows, int cols) {
        return new Matrix(rows, cols);
    }

    public int getNumberOfRows() {
        return rows;
    }

    public int getNumberOfCols() {
        return cols;
    }

    public double getValueAt(int r, int c) throws IllegalAccessException {
        if (r < 0 && r >= rows && c < 0 && c >= cols) {
            throw new IllegalAccessException();
        }
        return values[r][c];
    }

    public void setValueAt(int r, int c, double v) throws IllegalAccessException {
        if (r < 0 && r >= rows && c < 0 && c >= cols) {
            throw new IllegalAccessException();
        }
        values[r][c] = v;
    }

    public void fillRandom() {
        for (int r = 0; r < rows; ++r) {
            for (int c = 0; c < cols; ++c) {
                values[r][c] = (int) (Math.random() * 10);
            }
        }
    }

    public Matrix sum(Matrix a) {
        if (!(rows == a.rows && cols == a.cols)) {
            return null;
        }
        Matrix ret = createMatrix(this);
        for (int r = 0; r < rows; ++r) {
            for (int c = 0; c < cols; ++c) {
                ret.values[r][c] = values[r][c] + a.values[r][c];
            }
        }
        return ret;
    }

    public Matrix difference(Matrix a) {
        if (!(rows == a.rows && cols == a.cols)) {
            return null;
        }
        Matrix ret = createMatrix(this);
        for (int r = 0; r < rows; ++r) {
            for (int c = 0; c < cols; ++c) {
                ret.values[r][c] = values[r][c] - a.values[r][c];
            }
        }
        return ret;
    }

    public Matrix product(Matrix a) {
        //product defined only for n x n matrices
        final int n = rows;
        if (!(cols == rows && a.rows == n && a.cols == n)) {
            return null;
        }
        Matrix ret = createMatrix(this);

        for (int i = 0; i < n; ++i) {
            for (int j = 0; j < n; ++j) {
                ret.values[i][j] = 0.0;
                for (int k = 0; k < n; ++k) {
                    ret.values[i][j] += values[i][k] * a.values[k][j];
                }
            }
        }
        return ret;
    }

       public @Override
    String toString() {
        StringBuilder str = new StringBuilder();
        double[][] m = this.values;
        for (int r = 0; r < rows; ++r) {
            for (int c = 0; c < cols; ++c) {
                str.append(String.format("%s%4.1f", c > 0 ? " " : "", m[r][c]));
            }
            str.append("\n");
        }
        return str.toString();
    }

}
